""" Setup scripts for dozy. """
from setuptools import setup, find_packages


setup(
    name="dozy",
    version="0.0.4",
    description="A simple and user-friendly tool for operating images and videos",
    url="https://github.com/vra/dozy",
    author="Yunfeng Wang, Peisen Wang",
    author_email="wyf.brz@gmail.com",
    packages=find_packages()
)
